<?

ini_set('max_execution_time', 60);


class SegFault{
	var $file;
	var $fields=array(); // Fields list
	var $parser; // XML parser
	var $nesting; // Nesting level
	var $row;
	var $is_null;
	var $is_value;
	var $field_name;
	var $record=array();
	var $total_records;


	function SegFault(){
		$this->file=dirname(__FILE__) . '/segfault.xml';

		if(!is_readable($this->file)){
			die("Error: file not found: {$this->file}");
		}
		
		$this->add_field('COD');
		$this->add_field('REFPROV');
		$this->add_field('DESCRIP');
		$this->add_field('PRECIO');
		$this->add_field('DISP');
		$this->add_field('F_DISP');
		$this->add_field('DESCRIP_WEB');
		$this->add_field('DESCRIP_AMPL');
		$this->add_field('FABRICANTE');
		$this->add_field('EAN');
		$this->add_field('CATEGORIA');
		$this->add_field('OFERTA');
		$this->add_field('OBSOLETO');
		$this->add_field('PROMOCION');
		$this->add_field('BROKER');
		$this->add_field('IMAGEN');
		$this->add_field('IMAGEN_AMPL');
		$this->add_field('IMAGEN_REDU');
		$this->add_field('PLATAFORMA');
		$this->add_field('F_INI_PROMO');
		$this->add_field('F_FIN_PROMO');
		$this->add_field('F_FIN_OFERTA');
		$this->add_field('F_ALTA');
		$this->add_field('OBSERV_PROMO');
		$this->add_field('SUBCATEGORIA');
		$this->add_field('CANON');

		$this->start_parsing();
	}


	function add_field($name=NULL){
		if($name){
			$this->fields[]=strtoupper(trim($name));
		}
	}


	function start_parsing(){
		$this->nesting=0;
		$this->is_value=false;
		$this->is_null=false;

		$this->parser=xml_parser_create('ISO-8859-1');
		xml_set_object($this->parser, &$this);
		$this->total_records=0;
		xml_set_element_handler($this->parser, 'xml_start', 'xml_end');
		xml_set_character_data_handler($this->parser, 'xml_content');


		$fp=fopen($this->file, 'rb');
		if($fp){
			while($buffer=fread($fp, 4096)){
				if(!xml_parse($this->parser, $buffer, feof($fp))){
					die(sprintf("Error: XML error: line %d: %s", xml_get_current_line_number($this->parser), xml_error_string(xml_get_error_code($this->parser))));
				}
			}
		}else{
			die('Error: error reading file: ' . $this->file);
		}
		
		fclose($fp);
		xml_parser_free($this->parser);
	}
	
	
	 
	 
	function xml_start($parser, $name, $attributes){
		$this->nesting++;
		if(in_array($name, $this->fields)){ // Field starts
			$this->is_value=true;
			$this->field_name=$name;
			if(array_key_exists('NULL', $attributes)){
				if($attributes['NULL']=='TRUE'){
					$this->is_null=true;
					$this->record[$this->field_name]=NULL;
				}else{
					echo ('Warning: NULL attribute (' . $attributes['NULL'] . ') in tag (' . $name . '): TRUE was expected');
				}
			}
		}elseif($name=="ROW"){ // Row (=record) starts
			$this->record=null;
			
			if(array_key_exists('NUM', $attributes)){
				$this->row=$attributes['NUM'];
			}else{
				$this->row=NULL;
				echo ('Warning: Fila sin atributo NUM');
			}
			$this->record['NUM']=$this->row;
		}
	}


	function xml_content($parser, $dato){
		if($this->is_value){
			if(!$this->is_null){
				$this->record[$this->field_name].=$dato;
			}
		}
	}
	
		
	function xml_end($parser, $name){
		$this->nesting--;
		if(in_array($name, $this->fields)){ // Field ends
			$this->field_name=NULL;
		}elseif($name=="ROW"){ // Row (=record) ends: we can process it
			$this->row=NULL;
			$this->total_records++;
			
			echo "Record #{$this->total_records} processed\n";
		}
		$this->is_value=false;
		$this->is_null=false;
	}
}

new SegFault;

echo "OK: no segmentation fault happened\n";

?>